@@ -1,5 +1,4 @@
 import fsspec
-from pathlib import Path
 from appyter.ext.urllib import join_url
 from appyter.ext.tempfile import tempdir
 
@@ -10,7 +9,7 @@ def serve(app_path, **kwargs):
   logger = logging.getLogger(__name__)
   from subprocess import Popen
   from appyter.context import get_env, get_jinja2_env, find_blueprints, get_appyter_directory
-  config = get_env(**kwargs)
+  config = get_env(**kwargs, mode='prerender')
   logger.info(kwargs)
   env = get_jinja2_env(config=config)
   exit_code = 1
@@ -26,7 +25,7 @@ def serve(app_path, **kwargs):
       fw.write(render_form_from_nbtemplate(env, nbtemplate))
     with (tmp_dir/'index.json').open('w') as fw:
       import json
-      from appyter.render.nbinspect import render_nbtemplate_json_from_nbtemplate
+      from appyter.render.nbinspect.nbtemplate_json import render_nbtemplate_json_from_nbtemplate
       json.dump(render_nbtemplate_json_from_nbtemplate(env, nbtemplate), fw)
     with (tmp_dir/'landing.html').open('w') as fw:
       env.get_template('landing.j2').stream(
@@ -42,7 +41,7 @@ def serve(app_path, **kwargs):
         get_appyter_directory=get_appyter_directory,
         find_blueprints=find_blueprints,
       ).dump(fw)
-    logger.info(f"Starting production instance at http://{config['HOST']}:{config['PORT']}{config['PREFIX']} ...")
+    logger.info(f"Starting production instance at http://{config['HOST']}:{config['PORT']}{config['PREFIX']}/ ...")
     with Popen(['supervisord', '-n', '-c', str(tmp_dir/'supervisord.conf')]) as proc:
       try:
         exit_code = proc.wait()
@@ -50,4 +49,4 @@ def serve(app_path, **kwargs):
         proc.terminate()
         exit_code = proc.wait()
   #
-  sys.exit(exit_code)
+  return exit_code
